# Currently only contains the version identifier
# Examples of this style:
# - https://github.com/pypa/warehouse/blob/64ca42e42d5613c8339b3ec5e1cb7765c6b23083/warehouse/__about__.py
# - https://github.com/python-excel/xlrd/blob/master/xlrd/info.py

__version__ = '0.4b2'



# --- Hoshino ---
# Download product list
# Delete empty rows
# Save as CSV

import os.path

from requests import Session
import pandas as pd

from .supplier import Supplier  # , ScappamentoError


supplier_name = 'Hoshino'


def update():
    # Config
    key_list = [
        'user',
        'password',
        'login_url',
        'form_action_url',
        'report_url',
        'target_path',
        'csv_filename'
    ]
    hoshino = Supplier(supplier_name, key_list)

    print(hoshino)

    [user,
     password,
     login_url,
     form_action_url,
     report_url,
     target_path,
     csv_filename] = hoshino.val_list

    with Session() as s:
        # Login
        print('Logging in...')
        s.get(login_url)
        payload = {'email': user, 'password': password}
        s.post(form_action_url, data=payload)

        # Download
        print('Downloading...')
        r = s.get(report_url)

    # Cleanup
    print('Cleaning up...')
    r_clean = r.text.replace('Menil percussions', 'Meinl percussions')

    report = pd.read_json(r_clean)
    report.dropna(inplace=True)

    print('Saving...')
    csv_filepath = os.path.join(target_path, csv_filename)
    report.to_csv(csv_filepath, sep=';')


if __name__ == '__main__':
    pass  # will be test



# --- Proel ---
# WIP, will be used to merge 2 incomplete / conflicting CSV files

import pandas as pd

from .supplier import Supplier  # , ScappamentoError


supplier_name = 'Proel'


def update():
    # Config
    key_list = ['csv_filename_1',
                'csv_filename_2']
    proel = Supplier(supplier_name, key_list)

    print(proel)

    [csv_filename_1,
     csv_filename_2] = proel.val_list

    # Mapping header from csv2 to csv1
    headers_map = {'Product_name': 'PRODUCT_CODE',
                   'Cat_1': 'CAT_1',
                   'Lead Time': 'LEAD_TIME'}

    if headers_map:
        pass

    # note: file 2 is the "master" one
    csv_1 = pd.read_csv(csv_filename_1, sep=';', encoding='ISO-8859-1')
    csv_2 = pd.read_csv(csv_filename_2, sep=';', encoding='utf-8')

    # DONE: create header map between the two files
    # print(csv_1.iloc[:, 31].tolist()[1:50])
    # print(csv_1.iloc[:, 0].tolist()[1:50])
    print(csv_1.columns.tolist())

    csv1_product_code = csv_1['PRODUCT_CODE'].tolist()
    print('CSV1 entries: ', len(csv1_product_code))

    csv2_product_code = csv_2['Product_name'].tolist()
    print('CSV2 entries: ', len(csv2_product_code))

    common = [x for x in csv2_product_code if x in csv1_product_code]
    print('Common entries: ', len(common), '!')

    print(csv_2['Product_name'][3])

    # TODO: use DataFrame.join(), specify common key = Product_Name, retrieve list of column names


if __name__ == '__main__':
    update()



# --- Yamaha ---
# Read config file
# Log into B2B website
# Download Excel product list (no disk)
# Clean Excel table,convert to CSV, save

import io

from requests import session
import olefile
import pandas as pd

from .supplier import Supplier, ScappamentoError


supplier_name = 'Yamaha'


def update():
    # Credentials and URLs
    key_list = ['email',
                'password',
                'login_url',
                'form_action_url',
                'xls_url',
                'logout_url',
                # 'xls_filename',
                'csv_filename',
                'target_path',
                'expected_columns_len']
    yamaha = Supplier(supplier_name, key_list)

    print(yamaha)

    [email,
     password,
     login_url,
     form_action_url,
     xls_url,
     logout_url,
     # xls_filename,
     csv_filename,
     target_path,
     expected_columns_len] = yamaha.val_list

    with session() as s:
        # Login
        print("Logging in...")
        s.get(login_url)  # set preliminary cookies
        payload = {'email': email, 'password': password, 'submitform': 'Invia'}
        s.post(form_action_url, data=payload)

        # Download
        print("Downloading...")
        r = s.get(xls_url)

        # Logout
        s.get(logout_url)

    xls_file = io.BytesIO(r.content)
    ole = olefile.OleFileIO(xls_file)
    if not ole.exists('Workbook'):
        raise ScappamentoError("Fatal error: malformed xls file")

    wb = ole.openstream('Workbook')
    list_xls = pd.read_excel(wb, engine='xlrd', header=None)
    print(list_xls.head())

    # Check file content format
    if len(list_xls.columns) != int(expected_columns_len):  # check for usual header size
        raise ScappamentoError("Unexpected datasheet header size")

    # Edit, convert & save, delete original file
    print('Saving...')
    list_xls.drop([0, 1, 2, 3, 4, 5], inplace=True)
    list_xls.to_csv(target_path + csv_filename, sep=';', header=None, index=False, encoding='utf-8')


if __name__ == '__main__':
    update()



# --- Frenexport ---
# Read config file
# Connect to B2B database
# Execute query (no disk)
# Clean response, convert to CSV, save

import mysql.connector as msq
import pandas

from .supplier import Supplier  # , ScappamentoError


supplier_name = 'Frenexport'


def update():
    # Credentials and URLs
    key_list = ['host',
                'database',
                'user',
                'password',
                'sql_filename',
                'csv_filename',
                'target_path']
    frenexport = Supplier(supplier_name, key_list)

    print(frenexport)

    [host,
     database,
     user,
     password,
     sql_filename,
     csv_filename,
     target_path] = frenexport.val_list

    with open(target_path + sql_filename) as f:  # file with which to query the database
        query = f.read()

    # Database connection and query
    print('Connecting to database...')
    conn = msq.connect(host=host, database=database, user=user, password=password)
    print('Querying...')
    results = pandas.read_sql_query(query, conn)
    conn.close()

    # Result cleanup: separator, fields
    print('Cleaning up...')
    results_clean = results.replace(';', ',', regex=True)  # replace instances of to-be CSV separator character

    edited_row_count = 0
    edited_field_count = 0
    # TODO: info prints
    for i in range(0, len(results.index)):  # clean product "pretty name", remove instances of other fields' content
        des_art_temp = results_clean.at[i, 'DES_ART']  # cache fields instead of accessing dataframe x times
        marca_temp = results_clean.at[i, 'MARCA'].lower()  # lower() is tailored to query file currently in use
        modello_temp = results_clean.at[i, 'MODELLO'].lower()
        edited_field_count = edited_field_count + 1

        if marca_temp and marca_temp in des_art_temp:
            des_art_temp = des_art_temp.replace(' ' + marca_temp, '')  # update field for later checks
            results_clean.at[i, 'DES_ART'] = des_art_temp

        if modello_temp and modello_temp in des_art_temp:
            results_clean.at[i, 'DES_ART'] = des_art_temp.replace(' ' + modello_temp, '')
        elif modello_temp and modello_temp.replace('-', '') in des_art_temp:
            results_clean.at[i, 'DES_ART'] = des_art_temp.replace(' ' + modello_temp.replace('-', ''), '')

    results_clean.to_csv(target_path + csv_filename, sep=';', index=False)


if __name__ == '__main__':
    update()



# Convert CSV file from UCS-2 encoding to UTF-8

import os.path

from .supplier import Supplier


supplier_name = 'Mogar'


def update():
    key_list = ['csv_filename',
                'target_path']
    mogar = Supplier(supplier_name, key_list)

    print(mogar)

    [csv_filename,
     target_path] = mogar.val_list

    csv_file_path = os.path.join(target_path, csv_filename)
    csv_bak_path = csv_file_path + '.bak'

    if os.path.isfile(csv_bak_path):
        os.remove(csv_bak_path)

    os.rename(csv_file_path, csv_bak_path)

    with open(csv_bak_path, 'r', encoding='utf_16_le') as old_f:
        with open(csv_file_path, 'w', encoding='utf-8') as new_f:
            contents = old_f.read()
            new_f.write(contents)



# --- Fender ---
# Read config file
# Log into B2B website
# Download pre-made inventory Excel file (no disk)
# Download custom-selected "specs" Excel file (no disk)
# Convert both files to CSV

from requests import session
import pandas as pd

from .supplier import Supplier, ScappamentoError, browser_login


supplier_name = 'Fender'


def update():
    # Credentials and URLs
    key_list = ['email',
                'email_css_selector',
                'password',
                'password_css_selector',
                'butt_css_selector',
                'login_url',
                'xlsx_inventory_url',
                'xlsx_specs_url',
                'csv_inventory_filename',
                'csv_specs_filename',
                'target_path']
    fender = Supplier(supplier_name, key_list)

    print(fender)

    [email,
     email_css_selector,
     password,
     password_css_selector,
     butt_css_selector,
     login_url,
     xlsx_inventory_url,
     xlsx_specs_url,
     csv_inventory_filename,
     csv_specs_filename,
     target_path] = fender.val_list

    cookies = browser_login(login_url, email_css_selector, email, password_css_selector, password, butt_css_selector)

    with session() as s:
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 "
                          "(KHTML, like Gecko) Chrome/44.0.2403.157 Safari/537.36 "
                   }
        s.headers.update(headers)
        for cookie in cookies:  # port session cookies over
            c = {cookie['name']: cookie['value']}
            s.cookies.update(c)

        print('Downloading inventory...')
        r_inventory = s.get(xlsx_inventory_url)  # download inventory Excel file

        print('Downloading specs...')
        r_specs = s.get(xlsx_specs_url)  # download specs Excel file

        # TODO: https://dealer.fender.com/logout POST

    # Convert to CSV and save
    # TODO: check header size and column names
    inventory_list_xlsx = pd.read_excel(r_inventory.content, header=None, engine='openpyxl')
    inventory_list_xlsx.to_csv(target_path + csv_inventory_filename, sep=';', header=None, index=False)

    specs_list_xlsx = pd.read_excel(r_specs.content, header=None, engine='openpyxl')
    specs_list_xlsx.to_csv(target_path + csv_specs_filename, sep=';', header=None, index=False)


if __name__ == '__main__':
    update()



# --- Suonostore ---
# Read config file
# Download CSV file (no disk)
# Clean file, save

from requests import Session

from .supplier import Supplier, fix_illegal_sep_quotes, switch_sep, fix_illegal_inch  # , ScappamentoError


supplier_name = 'Suonostore'


def fix_timestamp(line):
    return line.replace('00:00:00', '').replace('/  /', '')


def fix_decimals(line):
    return line.replace('.00000,', ',').replace('.000,', ',').replace('.00,', ',')


def update():
    # Credentials and URLs
    key_list = ['user',
                'password',
                'csv_url',
                'csv_filename',
                'target_path']
    suonostore = Supplier(supplier_name, key_list)

    print(suonostore)

    [user,
     password,
     csv_url,
     csv_filename,
     target_path] = suonostore.val_list

    # Download
    with Session() as s:
        print('Downloading with auth...')
        # Site uses HTTP Basic Auth
        r = s.get(csv_url, auth=(user, password), headers={'User-Agent': 'Chrome'})

    # Cleanup: numbers, separators, dates, <inches> symbols
    sep = ';'
    new_csv = ''
    line_count = 0
    problematic_line_count = 0
    problematic_field_count = 0
    fixed_problematic_line_count = 0

    print('Cleaning up...')

    for line in r.content.decode(r.encoding).splitlines():  # each line in the CSV file

        if not line_count:  # skip first line = CSV header
            new_csv = switch_sep(line, ',', sep)  # initialize
            line_count = line_count + 1
            continue

        temp_line = fix_timestamp(line)
        temp_line = fix_decimals(temp_line)
        temp_line = temp_line.replace('", ', '"; ')  # hack
        temp_line = switch_sep(temp_line, ',', sep)

        temp_line, sep_modified = fix_illegal_sep_quotes(temp_line, sep, ',')  # fix illegal separators

        field_count = 0
        temp_cod_art = ''
        rebuilt_temp_line = ''
        found_problematic_field = False
        match = False
        for field in temp_line.split(';'):  # for each field in line: look for double quotes as <inches> symbols

            if field_count == 1:  # if second field
                temp_cod_art = field

            if field.count('"') % 2:  # if double quotes parity check fails
                found_problematic_field = True
                [temp_field, match] = fix_illegal_inch(field)
                rebuilt_temp_line = rebuilt_temp_line + sep + temp_field

                if not match:  # problematic fields are copied as-is for now
                    print('⚠ [ Row ', line_count, '][', temp_cod_art, ']', 'Uh oh: field ', field_count + 1)
                    problematic_field_count = problematic_field_count + 1
                    rebuilt_temp_line = rebuilt_temp_line + sep + field

            else:
                if not field_count:
                    rebuilt_temp_line = field
                else:
                    rebuilt_temp_line = rebuilt_temp_line + sep + field

            field_count = field_count + 1

        if found_problematic_field or sep_modified:  # if intervention was necessary
            if match or sep_modified:  # if a solution was found
                fixed_problematic_line_count = fixed_problematic_line_count + 1
            problematic_line_count = problematic_line_count + 1

        new_csv = new_csv + '\n' + rebuilt_temp_line.strip()
        line_count = line_count + 1

    print('⚠ ' if problematic_field_count else '✓ ',
          problematic_field_count, ' problematic field', '' if problematic_field_count == 1 else 's',
          ' in ', problematic_line_count - fixed_problematic_line_count, ' problematic line',
          '' if problematic_line_count - fixed_problematic_line_count == 1 else 's',
          ' (', problematic_line_count, ' lines total, ', fixed_problematic_line_count, ' fixed)', sep='')

    with open(target_path + csv_filename, 'w', encoding='utf-8') as f:
        f.write(new_csv)


if __name__ == '__main__':
    update()



# Common functionality to be shared across modules
# This is not a stand-alone module
# Supplier class for init stuff

import configparser

import chromedriver_autoinstaller
from selenium import webdriver

default_config_path = 'C:\\Ready\\ReadyPro\\Archivi\\scappamento_config\\'
default_config_name = 'scappamento.ini'


class Supplier:
    name = ''
    val_list = []

    def __init__(self, name, key_list=None, config_path=None):
        self.name = name
        if key_list:
            if config_path:
                self.load_config(key_list, config_path)
            else:
                self.load_config(key_list)

    def __str__(self):
        return '-- ' + self.name + ' --\n'

    def load_config(self, key_list, config_path=default_config_path+default_config_name):
        config = configparser.ConfigParser()

        with open(config_path) as f:
            config.read_file(f)

            for key in key_list:
                self.val_list.append(config[self.name][key])


class ScappamentoError(Exception):
    pass


def fix_illegal_inch(field):
    match = False
    for i in range(len(field) - 1, -1, -1):  # for each char in field, inverted, greedy [0-9]" match
        if i and field[i] == '"' and field[i - 1].isdigit():
            match = True
            return field[0:i] + '″' + field[i + 1:len(field)], match  # greedy
        else:
            return field, match


# scan line by double-quotes pairs
# look for separator characters inside quote pairs
# replace separator with sub
# return fixed line, is_modified
def fix_illegal_sep_quotes(line, sep, sep_replacement):
    is_modified = False
    in_quotes = False
    new_line = ''
    field_start = 0
    for count, char in enumerate(line):
        if char == '"':
            in_quotes = not in_quotes  # toggle

            if in_quotes:
                field_start = count+1

            if not in_quotes and line[count+1] != sep:  # the "closing" double quote is doing something else
                in_quotes = True  # reset to correct value

                new_line = new_line + char

                field = new_line[field_start:count+1]
                new_field, match = fix_illegal_inch(field)

                new_line = new_line.replace(field, new_field)
                continue
            else:
                new_line = new_line + char
                continue

        if in_quotes and char == sep:
            new_line = new_line + sep_replacement
            is_modified = True
        else:
            new_line = new_line + char

    return new_line, is_modified


# change from one separator character to another
def switch_sep(line, sep_old, sep_new):
    return line.replace(sep_old, '%temp%').replace(sep_new, sep_old).replace('%temp%', sep_new)  # flip old and new


# login into AJAX websites
def browser_login(login_url, user_css_selector, user, password_css_selector, password, butt_css_selector,
                  pop_css_selector=None):
    chromedriver_autoinstaller.install()

    options = webdriver.ChromeOptions()
    options.add_argument('--headless')
    with webdriver.Chrome(options=options) as driver:
        # Login
        driver.get(login_url)

        if pop_css_selector:
            pop_up_butt = driver.find_element_by_css_selector(pop_css_selector)
            pop_up_butt.click()

        user_input = driver.find_element_by_css_selector(user_css_selector)
        user_input.send_keys(user)

        pass_input = driver.find_element_by_css_selector(password_css_selector)
        pass_input.send_keys(password)

        login_butt = driver.find_element_by_css_selector(butt_css_selector)
        login_butt.click()

        return driver.get_cookies()



# User-facing CLI
# Top-level script for scappamento usage

import sys
import argparse

from scappamento import yamaha
from scappamento import fender
from scappamento import frenexport
from scappamento import suonostore
from scappamento import mepa
from scappamento import mogar
# from scappamento import daddario
from scappamento import hoshino


def main():
    suppliers_list = [
        'yamaha',
        'fender',
        'frenexport',
        'suonostore',
        'mepa',
        'mogar',
        # 'daddario',
        'hoshino'
    ]
    updaters_list = [
        yamaha.update,
        fender.update,
        frenexport.update,
        suonostore.update,
        mepa.update,
        mogar.update,
        # daddario.update,
        hoshino.update
    ]
    suppliers_help = 'the supplier to be updated'

    parser = argparse.ArgumentParser(description='Automate B2B provisioning.')
    parser.add_argument('supplier_name', choices=suppliers_list, help=suppliers_help)

    namespace = parser.parse_args()

    i = 0
    for supplier in suppliers_list:
        if namespace.supplier_name == supplier:
            updaters_list[i]()
        i = i + 1


if __name__ == '__main__':
    sys.exit(main())



import setuptools

from scappamento.__about__ import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name='scappamento',
    version=__version__,
    author='Lorenzo Bunino',
    author_email="bunino.lorenzo@gmail.com",
    description="B2B automation for music stores",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lorenzobunino/scappamento",
    packages=setuptools.find_packages(),  # TODO: find_packages vs hand-compiled list
    entry_points={
        'console_scripts': [
            'scappamento = scappamento.__main__:main'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",  # TODO: fix path handling and
                                                     #  change "Microsoft :: Windows" to "OS Independent"
    ],
    install_requires=[
        'pandas>=1.1.2',
        'xlrd>=2.0.1',
        'xlutils>=2.0.0',
        'requests~=2.24.0',
        'selenium~=3.141.0',
        'beautifulsoup4~=4.9.3',
        'mysql-connector-python~=8.0.11',
        'chromedriver-autoinstaller>=0.2.2',
        'xlwt>=1.3.0',
        'olefile>=0.46'
    ],
    python_requires='>=3.6'
)



# --- MusicPool ---
# Read config file
# Log into B2B website
# Download Excel file into dataframe (no disk)
# Check column count
# Clean file content
# Convert to CSV

import sys
import re

from requests import session
from bs4 import BeautifulSoup
import pandas as pd

from .supplier import Supplier  # , ScappamentoError


supplier_name = 'MusicPool'


def update():
    # Credentials and URLs
    key_list = ['email',
                'password',
                'login_url',
                'form_action_url',
                'csv_filename',
                'target_path',
                'expected_columns_len']
    musicpool = Supplier(supplier_name, key_list)

    print(musicpool)

    [email,
     password,
     login_url,
     form_action_url,
     csv_filename,
     target_path,
     expected_columns_len] = musicpool.val_list

    with session() as s:
        # Login
        s.get(login_url)  # set preliminary cookies
        payload = {'email': email, 'passwd': password, 'SubmitLogin': ''}
        s.post(form_action_url, data=payload, headers={'User-Agent': 'Chrome'})

        # Parse
        musicpool_soup = BeautifulSoup(s.get(login_url).text, 'html.parser')
        intermediate_url = musicpool_soup.find(text='DOWNLOAD').parent.parent['href']

        external_host_soup = BeautifulSoup(s.get(intermediate_url).text, 'html.parser')
        # mark regular expression as a raw string to prevent <invalid unicode escape sequence> errors
        xlsx_url = external_host_soup.find(text=re.compile(r'Download[\s]*\([\d]+[.]?[\d]*[A-Z][A-Z]\)')).parent['href']

        xlsx_list = pd.read_excel(xlsx_url, header=None)

        # Check file format
        if len(xlsx_list.columns) != expected_columns_len:  # check for usual header size
            print("Unexpected datasheet header size")
            sys.exit()

        xlsx_list.drop([1], inplace=True)
        xlsx_list.to_csv(target_path + csv_filename, sep=';', index=False, header=None)


if __name__ == '__main__':
    update()



# --- D'Addario ---
# Download product list
# Perform file conversion
# Maybe cleanup
# Save as CSV

import os.path

from requests import Session
from bs4 import BeautifulSoup
import pandas as pd

from .supplier import Supplier  # , ScappamentoError


supplier_name = 'D\'Addario'


def update():
    # Config
    key_list = [
        'email',
        'password',
        'login_url',
        'form_action_url',
        'return_url',
        'token_input_css',
        'dl_form_action_url',
        'target_path',
        'csv_filename'
    ]
    daddario = Supplier(supplier_name, key_list)

    print(daddario)

    [email,
     password,
     login_url,
     form_action_url,
     return_url,
     token_input_css,
     dl_form_action_url,
     target_path,
     csv_filename] = daddario.val_list

    with Session() as s:
        # Login
        print('Logging in...')
        s.get(login_url)
        payload = {'Username': email, 'Password': password, 'ReturnUrl': return_url}
        r = s.post(form_action_url, data=payload)

        print('Downloading...')
        daddario_soup = BeautifulSoup(r.text, 'html.parser')
        token_input = daddario_soup.select_one(token_input_css)
        payload = {token_input['name']: token_input['value']}
        r = s.post(dl_form_action_url, data=payload)

    list_xls = pd.read_excel(r.content, header=None)

    print('Saving...')
    list_xls.to_csv(os.path.join(target_path, csv_filename), sep=';', header=None, index=False)


if __name__ == '__main__':
    update()



# --- MEPA ---
# Read config file
# Read Excel file generated by Ready Pro
# Copy rows into Excel file generated by acquistinretepa
# Save
# Upload file to acquistinretepa portal (sostituzione integrale)

import os.path
import time

from xlrd import open_workbook
from xlutils.copy import copy
from xlutils.save import save
import chromedriver_autoinstaller
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.common.by import By

from .supplier import Supplier  # , ScappamentoError


supplier_name = 'MEPA'


def update():
    # Config, filenames
    key_list = ['readypro_excel_filename',  # TODO: more consistency in <filename> vs <fully qualified path> stuff
                'mepa_excel_filename',
                'new_excel_filename',
                'target_path',
                'login_url',
                'user',
                'user_css',
                'password',
                'password_css',
                'login_butt_css',
                'login_popup_css',
                'file_input_css',
                'upload_butt_css',
                'catalogo_css',
                'sostituzione_css']
    mepa = Supplier(supplier_name, key_list)

    print(mepa)  # Title

    [readypro_excel_filename,
     mepa_excel_filename,
     new_excel_filename,
     target_path,
     login_url,
     user,
     user_css,
     password,
     password_css,
     login_butt_css,
     login_popup_css,
     file_input_css,
     upload_butt_css,
     catalogo_css,
     sostituzione_css] = mepa.val_list

    # TODO: add some info prints
    # Open and re-save excel file to fix mildly malformed files
    readypro_xls = open_workbook(readypro_excel_filename)
    save(readypro_xls, readypro_excel_filename+'.f')

    # Copy data from generated spreadsheet to downloaded to-edit one
    readypro_xls = open_workbook(readypro_excel_filename+'.f')
    mepa_xls_model = open_workbook(mepa_excel_filename, formatting_info=True)

    mepa_xls_new = copy(mepa_xls_model)

    mepa_sheet = mepa_xls_new.get_sheet(1)
    readypro_sheet = readypro_xls.sheet_by_index(0)  # get_sheet() throws an error

    # TODO: avoid empty rows when an invalid entry is skipped
    for i in range(1, readypro_sheet.nrows):  # rows (skip header)
        for j in range(0, 20):  # cols
            marca = False
            for m in range(14, 18):  # Marca (A-D) = campo 14 (14 0-based with offset)
                marca = marca or readypro_sheet.cell_value(i, m)

            if marca and readypro_sheet.cell_value(i, 1):  # CodArt Produttore = campo 1
                mepa_sheet.write(i, j, readypro_sheet.cell_value(i, j+1))  # offset read, column 0 is empty

    mepa_xls_new.save(os.path.join(target_path, new_excel_filename))

    chromedriver_autoinstaller.install()

    options = webdriver.ChromeOptions()
    options.add_argument('--headless')
    options.add_argument('--window-size=3840,2160')
    with webdriver.Chrome(options=options) as driver:
        # Login
        driver.get(login_url)

        pop_up_butt = driver.find_element_by_css_selector(login_popup_css)
        pop_up_butt.click()

        user_input = driver.find_element_by_css_selector(user_css)
        user_input.send_keys(user)

        pass_input = driver.find_element_by_css_selector(password_css)
        pass_input.send_keys(password)

        login_butt = driver.find_element_by_css_selector(login_butt_css)
        login_butt.click()

        catalogo_butt = WebDriverWait(driver, 5).until(ec.element_to_be_clickable((By.CSS_SELECTOR, catalogo_css)))
        catalogo_butt.click()

        sost_butt = WebDriverWait(driver, 5).until(ec.presence_of_element_located((By.CSS_SELECTOR, sostituzione_css)))
        sost_butt.click()

        file_input = WebDriverWait(driver, 10).until(ec.presence_of_element_located((By.CSS_SELECTOR, file_input_css)))
        file_input.send_keys(os.path.join(target_path, new_excel_filename))

        upload_butt = WebDriverWait(driver, 5).until(ec.presence_of_element_located((By.CSS_SELECTOR, upload_butt_css)))
        upload_butt.click()

        time.sleep(10)  # dirty, switch with something that checks portal output message


if __name__ == '__main__':
    update()
